#!/usr/bin/perl -I/usr/sausalito/perl
# $Id: import_uconf.pl, v1.1.0.0 Tue 08 Apr 2008 11:03:18 AM CEST mstauber Exp $
# Copyright 2006-2008 Solarspeed Ltd. All rights reserved.

# This constructor reads /etc/vz/uconf/000_customers.conf and updates or generates the user 
# information located in CCE accordingly. This is especially important for clusters on fail overs, 
# so that the user information can be inherited on the new primary after a fail-over.

$whatami = "constructor";
#$whatami = "handler";

use CCE;
my $cce = new CCE;
my $conf = '/var/lib/cobalt';

if ($whatami eq "handler") {
    $cce->connectfd();
}
else {
    $cce->connectuds();
}

# Location of the config file:
$customers_Config = "/etc/vz/uconf/000_customers.conf";
$customers_Dir = "/etc/vz/uconf";

# Config file present?
if (-e $customers_Config) {
	#&del_customers;
        &dir_read;
}
else {
    unless ( -e "/etc/vz/conf/0.conf") {
	# Ok, something is amiss. Are we a cluster?
	if ((-e "/etc/drbd.conf") && (-e "/etc/ha.d/ha.cf")) {

	    # Houston, we have a problem: We're a cluster, but we're secondary.
	    # We need to delete all customers for now:
	    &del_customers;
	}
    }
}

sub dir_read {
    open (F, $customers_Config) || die "Could not open $customers_Config: $!";
    $a = 0;                             # Matrix line number, whereas 0 is the first (useable) line of the vzlist dump
    while ($line = <F>) {
        chomp($line);
        next if $line =~ /^\s*$/;               # skip blank lines
        if ($line =~ /^([A-Za-z_]\w*)/) {
            #$matrix_name = $1;
            $matrix_name = 'CONFIG';
            (@row) = split (/:/, $line);
            push (@{$matrix_name}, \@row);
            if ($CONFIG[$a][0] ne "") {
                # Found user w/o owned VPS:
		$username = $CONFIG[$a][0];
		&read_userinfo($username);
            }
	    elsif (($CONFIG[$a][0] ne "") && ($CONFIG[$a][1] ne "")) {
                # Found user with owned VPS(s):
		$username = $CONFIG[$a][0];
		&read_userinfo($username);
	    }
        }
        $a++;
    }
    close(F);
}

sub read_userinfo {
    $username = $_[0];
    $uconf_user_file = "$customers_Dir" . "/" . $username . ".uconf";
    open (U, $uconf_user_file) || die "Could not open $uconf_user_file: $!";
    $b = 0;
    while ($cline = <U>) {
	if ($cline =~ /^customer_comment:(.*)/) {
		$USER[$username][31] = $2;
	}
    	elsif ($cline =~ /^(.*):(.*)/) {
    		$key = $1;
		$value = $2;
        	$USER[$username][$b] = $2;
		$b++;
    	}
	else {
		$USER[$username][31] = $USER[$username][31] . $cline;
	}
    }
    close(U);

    @oids = $cce->find('User', {'name' => $username});
    if ($#oids < 0) {
        # Object not yet in CCE. Creating new one.

        ($ok) = $cce->create('User', {
            'name' => $username,
            'capLevels' => "&ManageVPS&",
            'capabilities' => "&ManageVPS&",
            'customer_email' => $USER[$username][3],
            'uiRights' => $USER[$username][4],
            'crypt_password' => $USER[$username][5],
            'desc_readonly' => $USER[$username][6],
            'customer_city' => $USER[$username][7],
            'customer_phone' => $USER[$username][8],
            'volume' => $USER[$username][9],
            'customer_zip' => $USER[$username][10],
            'md5_password' => $USER[$username][11],
            'systemAdministrator' => $USER[$username][12],
            'customer_fax' => $USER[$username][13],
            'enabled' => $USER[$username][14],
            'customer_country' => $USER[$username][15],
            'description' => $USER[$username][16],
            'customer_owned_vps' => $USER[$username][17],
            'emailDisabled' => $USER[$username][18],
            'localePreference' => $USER[$username][19],
            'customer_address' => $USER[$username][20],
            'noFileCheck' => $USER[$username][21],
            'sortName' => $USER[$username][22],
            'fullName' => $USER[$username][23],
            'customer_state' => $USER[$username][24],
            'shell' => "/bin/false",
            'customer_company' => $USER[$username][26],
            'site' => $USER[$username][27],
            'stylePreference' => $USER[$username][28],
            'ui_enabled' => $USER[$username][29],
            'password' => $USER[$username][30],
            'customer_comment' => $USER[$username][31]
        });


	use Vz::User qw(usermod);
	$md5_pw = $USER[$username][11];
	# set admin's root password
	($ok) = usermod({ 'name' => $username, 'password' => $md5_pw });
    	&give_vps_to_owner;
        ($sys_oid) = $cce->find('User', {'name' => $username});
        ($ok, $sys) = $cce->get($sys_oid);
        ($ok) = $cce->set($sys_oid, '',{        # beware: CLASS vs. Namespace (see schema)! Our NameSpace is empty, hence: ''
            'md5_password' => $USER[$username][11],
            'password' => $USER[$username][30]
        });

    }
    else {
        # Object already present in CCE. Updating it.
        ($sys_oid) = $cce->find('User', {'name' => $username});
        ($ok, $sys) = $cce->get($sys_oid);

        ($ok) = $cce->set($sys_oid, '',{        # beware: CLASS vs. Namespace (see schema)! Our NameSpace is empty, hence: ''
            'name' => $username,
            'capLevels' => "&ManageVPS&",
            'capabilities' => "&ManageVPS&",
            'customer_email' => $USER[$username][3],
            'uiRights' => $USER[$username][4],
            'crypt_password' => $USER[$username][5],
            'desc_readonly' => $USER[$username][6],
            'customer_city' => $USER[$username][7],
            'customer_phone' => $USER[$username][8],
            'volume' => $USER[$username][9],
            'customer_zip' => $USER[$username][10],
            'md5_password' => $USER[$username][11],
            'systemAdministrator' => $USER[$username][12],
            'customer_fax' => $USER[$username][13],
            'enabled' => $USER[$username][14],
            'customer_country' => $USER[$username][15],
            'description' => $USER[$username][16],
            'customer_owned_vps' => $USER[$username][17],
            'emailDisabled' => $USER[$username][18],
            'localePreference' => $USER[$username][19],
            'customer_address' => $USER[$username][20],
            'noFileCheck' => $USER[$username][21],
            'sortName' => $USER[$username][22],
            'fullName' => $USER[$username][23],
            'customer_state' => $USER[$username][24],
            'shell' => "/bin/false",
            'customer_company' => $USER[$username][26],
            'site' => $USER[$username][27],
            'stylePreference' => $USER[$username][28],
            'ui_enabled' => $USER[$username][29],
            'password' => $USER[$username][30],
            'customer_comment' => $USER[$username][31]
        });
    	&give_vps_to_owner;
    }
}

sub give_vps_to_owner {
    # Ownership of VPS's is given to the respective owners here:
    $customer_owned_vps = $USER[$username][17];
    (@vps) = split (/&/, $customer_owned_vps);
    foreach $vpsentry (@vps) {
	if (($vpsentry ne "&") && ($vpsentry ne "")) {
            ($sys_oid_vps) = $cce->find('Vserver', {'vpsid' => $vpsentry});
	    ($ok, $vps_sys) = $cce->get($sys_oid_vps);
    	    ($ok) = $cce->set($sys_oid_vps, '',{        # beware: CLASS vs. Namespace (see schema)! Our NameSpace is empty, hence: ''
        	'name' => $username
    	    });
	}
    }
}

sub del_customers {
    @oids = $cce->find('User', {'capabilities' => 'ManageVPS'});
    foreach $oid (@oids) {
	($ok) = $cce->destroy($oid);
    }
}

$cce->bye('SUCCESS');
exit(0);

